(function($) {

    if ($("#leads5050_visit_map").length > 0) {
        leadsLoadVisitMap();
    }

    //define pool of points to use - this will be stored as a WP option
    let myArr = [-10, 10];


    function leadsLoadVisitMap() {
        let refreshTime = 7; //seconds
        let updateTime = 2; //minutes

        if (document.getElementById("leads5050_visit_map")){
            //define the start time for loading the map
            var startUpdate = Date.now();
            console.log('MAP AREA DETECTED', 'Load in progress');
            //load a list of location values from the server
            leadsFetchVisitData();

            //------------------------------------ Set Map Controls ------------------------------------
            const fullScreenControl = new ol.control.FullScreen();
            const overviewMapControl = new ol.control.OverviewMap({
                collapsed: false,
                layers: [
                    new ol.layer.Tile({
                        source: new ol.source.OSM()
                    })
                ]
            })
            const zoomSliderContol = new ol.control.ZoomSlider();
            const zoomToExtentContol = new ol.control.ZoomToExtent();
            let attributionControl = new ol.control.Attribution({
                collapsible: true
            });

            const tileLayer = new ol.layer.Tile({
                source: new ol.source.OSM({
                    attributions: [ ol.source.OSM.ATTRIBUTION, 'Data courtesy  <a href="https://leads5050.com/">Leads5050.com</a>' ],
                }),
                zIndex: 0,
                visible: true,
                opacity: 1
            })

            const map = new ol.Map({
                view: new ol.View({
                    center: [0, 0],
                    zoom: 2,
                    maxZoom: 11,
                    minZoom: 1
                }),
                layers: [tileLayer],
                target: "leads5050_visit_map",
                keyboardEventTarget: document,
                controls: ol.control.defaults({attribution: false, zoom: false}).extend([
                    attributionControl,
                    fullScreenControl,
                    overviewMapControl,
                    zoomSliderContol,
                    zoomToExtentContol
                ])
            });
            //console.log(map);

            //Layer Group
            // const layerGroup = new ol.layer.Group({
            //     layers: [
            //         new ol.layer.Tile({
            //             source: new ol.source.OSM({
            //                 url: 'https://{a-c}.tile.openstreetmap.fr/hot/{z}/{x}/{y}.png'
            //             }),
            //             zIndex: 1,
            //             visible: false,
            //             //extent: [12269470.46160356, -5744185.936924426, 17359145.823643737, -852167.8704974577],
            //             opacity: 0.1
            //         })
            //     ]
            // });
            // map.addLayer(layerGroup);

            const vectorSource = new ol.source.Vector({
                wrapX: false,
            });

            const vectorLayer = new ol.layer.Vector({
                source: vectorSource,
                style: new ol.style.Style({
                    image: new ol.style.Circle({
                        radius: 5,
                        fill: new ol.style.Fill({color: [245, 156, 66, 1]}),
                        stroke: new ol.style.Stroke({
                            color: [245, 188, 66, 1],
                            width: 1
                        })
                    })
                })
            });
            map.addLayer(vectorLayer);
            $("#leads5050_map_status").html("Map loaded - Press ctrl+F5 if it doesn't display properly the first time");

            //------------------------------------ Position Pop-Up ------------------------------------
            // const popupContainerElement = document.getElementById('popup-coordinates');
            // const popup= new ol.Overlay({
            //     element: popupContainerElement,
            //     positioning: "center-left"
            // });
            //
            // map.addOverlay(popup);
            // map.on('click', function(e){
            //     //console.log(e.coordinate);
            //     var clickpos = e.coordinate;
            //     popup.setPosition(undefined);
            //     popup.setPosition(clickpos);
            //     popupContainerElement.innerHTML = clickpos;
            // });


            //------------------------------------ Other ------------------------------------
            map.on('click', function(e){
                console.log(ol.proj.toLonLat(e.coordinate));
            });

            function leadsAddLatestVisits(minElements= 3) {
                //if refresh time then update the array and reset the start time
                let myElapsed = Date.now()-startUpdate;
                if (myElapsed>(updateTime*60*1000)){
                    //get the map data and put this in myArr
                    leadsFetchVisitData();
                    startUpdate = Date.now();
                    console.log('VISITS DATA UPDATED', myArr);
                }

                //add or remove features from the chart
                if (myArr.length >= minElements){
                    //set the maximum number of elements to show on the chart
                    let maxFeatures = Math.ceil(0.7 * myArr.length);
                    //get the new point and add it to the vector layer
                    let ix = Math.floor(Math.random() * myArr.length);
                    let geom = new ol.geom.Point(ol.proj.fromLonLat(myArr[ix]));
                    let myFeature = new ol.Feature({
                        geometry: geom,
                    });
                    myFeature.setId('X' + Date.now());
                    vectorSource.addFeature(myFeature);
                    //remove the first point from the display
                    let allFeatures = vectorSource.getFeatures();
                    // console.log (allFeatures);
                    console.log ('There are ' + allFeatures.length + ' feature(s) at elapsed time ' + myElapsed);
                    //identify the last feature for removing
                    if (allFeatures[maxFeatures]!=undefined){
                        //console.log (allFeatures[2]);
                        let myID = allFeatures[0].id_;
                        //console.log (myID);
                        let featureToRemove = vectorSource.getFeatureById(myID);
                        vectorSource.removeFeature(featureToRemove);
                    }
                } else {
                    console.log ('No Points to Show');
                    $("#leads5050_map_status").html("<span class='leads5050_alert'>Gathering data - " +
                        "nothing to show yet. Please allow up to 2 hours for new sites.</span>");
                }
            }

            var duration = 15000;
            function leadsFlashFeature(feature) {
                var start = new Date().getTime();
                var listenerKey = tileLayer.on('postrender', animate);
                function animate(event) {
                    var vectorContext = ol.render.getVectorContext(event);
                    var frameState = event.frameState;
                    var flashGeom = feature.getGeometry().clone();
                    var elapsed = frameState.time - start;
                    var elapsedRatio = elapsed / duration;
                    // radius will be 5 at start and 30 at end.
                    var radius = ol.easing.easeOut(elapsedRatio) * 25 + 5;
                    var opacity = ol.easing.easeOut(1 - elapsedRatio);
                    var style = new ol.style.Style({
                        image: new ol.style.Circle({
                            radius: radius,
                            stroke: new ol.style.Stroke({
                                color: 'rgba(255, 0, 0, ' + opacity + ')',
                                width: 0.25 + opacity,
                            }),
                        }),
                    });
                    vectorContext.setStyle(style);
                    vectorContext.drawGeometry(flashGeom);
                    if (elapsed > duration) {
                        ol.Observable.unByKey(listenerKey);
                        return;
                    }
                    // tell OpenLayers to continue postrender animation
                    map.render();
                }
            }

            vectorSource.on('addfeature', function (e) {
                leadsFlashFeature(e.feature);
            });

            window.setInterval(leadsAddLatestVisits, (refreshTime*1000));
        }
    }

    /**
     * Fetch the visit data from the parent site
     */
    function leadsFetchVisitData(){
        let visitArr = [];
        if (varzl.hash.length > 5) {
            var api = $("#leads5050-api-key-value").text();
            if (api.length == 5){
                var api_url = 'https://leads5050.com/wp-json/ctasm/v1/lox/' + varzl.hash + '/' + btoa(api) + '/';
                console.log('REST Data API', api_url);
                $.ajax({
                    url: api_url,
                    method: 'GET',
                    success: function (rst_data) {
                        visitArr = rst_data;
                        if (visitArr.loc != null){
                            //convert the visitArr.loc object to an array and then verify the data
                            let dat = Object.values(visitArr.loc);
                            myArr = leadsVerifyData(dat);
                            $("#leads5050_map_status").html("Points last retrieved on " + (new Date()));
                        } else {
                            console.log('WARNING', 'There is no data to display');
                            $("#leads5050_map_status").html("No data to display on " + (new Date()));
                        }
                        // console.log('data retrieved', myArr);
                    },
                    error: function (xhr) {
                        console.log('ERROR', ('REST call error occurred: ' + xhr.status + ' ' + xhr.statusText));
                        $("#leads5050_map_status").html("There was a problem recovering data for this site");
                    }
                });
            } else {
                console.log('ERROR', 'API key not set');
                $("#leads5050_map_status").html("Please register the API by clicking the Setup button or press 'Save Changes'.");
            }
        } else {
            console.log('ERROR', 'Missing varzl (API Seed value)');
        }
    }

    function leadsVerifyData(incomingArr){
        let checkedArr = [];
        if (Array.isArray(incomingArr)){
            incomingArr.forEach(function(itm, ix){
                if (itm != null){
                    if(itm[0]>=-180 && itm[0]<=180){
                        if(itm[1]>=-90 && itm[1]<=90){
                            checkedArr.push(itm)
                        }
                    }
                }
            })
        }
        return checkedArr
    }

})(jQuery);
